#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const LibPath = require("path");
const LibFs = require("mz/fs");
const program = require("commander");
const lib_1 = require("./lib/lib");
const template_1 = require("./lib/template");
const pkg = require('../../package.json');
program.version(pkg.version)
    .parse(process.argv);
class DockerfileBuilderCLI {
    static instance() {
        return new DockerfileBuilderCLI();
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._validate();
            yield this._create();
            console.log('InitCLI complete.');
        });
    }
    /**
     * 验证参数，数据，环境是否正确
     *
     * @returns {Promise<void>}
     * @private
     */
    _validate() {
        return __awaiter(this, void 0, void 0, function* () {
            const jsonStat = yield LibFs.stat(LibPath.join(process.cwd(), 'Dockerfile.json'));
            if (!jsonStat.isFile()) {
                throw new Error('Dockerfile.json not found!');
            }
        });
    }
    /**
     * 创建Dockerfile
     *
     * @returns {Promise<void>}
     * @private
     */
    _create() {
        return __awaiter(this, void 0, void 0, function* () {
            const configs = require(LibPath.join(process.cwd(), 'Dockerfile.json'));
            this._fullShellPackage = new Map();
            yield this._recursionShellDependencies(configs.shell.store, configs.shell.packages);
            template_1.TplEngine.registerHelper('lcfirst', lib_1.lcfirst);
            let DockerfileContent = template_1.TplEngine.render('Dockerfile', {
                alpine: configs.alpine,
                apk: configs.apk,
                npm: configs.npm,
                shell: Array.from(this._fullShellPackage.values()),
                dockerfile: configs.dockerfile,
            });
            yield LibFs.writeFile(LibPath.join(process.cwd(), 'Dockerfile'), DockerfileContent);
        });
    }
    _recursionShellDependencies(store, packages) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let key in packages) {
                const [name, version] = packages[key].split('@');
                const shellPath = LibPath.join(process.cwd(), store, name, version);
                const shellDependenciesPath = LibPath.join(shellPath, 'dependencies.json');
                if (LibFs.existsSync(shellDependenciesPath) && LibFs.statSync(shellDependenciesPath).isFile()) {
                    yield this._recursionShellDependencies(store, require(shellDependenciesPath));
                    this._compareShellVersion({
                        name: name,
                        version: version,
                        path: lib_1.calcRelativeDir(LibPath.relative(process.cwd(), shellPath))
                    });
                }
                else {
                    this._compareShellVersion({
                        name: name,
                        version: version,
                        path: lib_1.calcRelativeDir(LibPath.relative(process.cwd(), shellPath))
                    });
                }
            }
        });
    }
    _compareShellVersion(newShellPackage) {
        if (!this._fullShellPackage.has(newShellPackage.name)) {
            this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            return;
        }
        const oldShellPackage = this._fullShellPackage.get(newShellPackage.name);
        let [nextMajor, nextMinor, nextPatch] = newShellPackage.version.split('.');
        let [curMajor, curMinor, curPatch] = oldShellPackage.version.split('.');
        if (nextMajor > curMajor) {
            this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            return;
        }
        if (nextMajor == curMajor) {
            if (nextMinor < curMinor || nextMinor == curMinor && nextPatch < curPatch) {
                // 依赖版本低于当前版本，不处理
            }
            else {
                this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            }
        }
    }
}
exports.DockerfileBuilderCLI = DockerfileBuilderCLI;
DockerfileBuilderCLI.instance().run().catch((err) => {
    console.log('error:', err.message);
});
