#!/usr/bin/env node

import * as LibPath from 'path';
import * as LibFs from 'mz/fs';
import * as program from 'commander';
import {calcRelativeDir, lcfirst} from './lib/lib';
import {TplEngine} from './lib/template';

const pkg = require('../../package.json');

program.version(pkg.version)
    .parse(process.argv);

interface DockerConfigs {
    alpine: string;
    apk: Array<string>;
    npm: Array<string>;
    shell: {
        store: string;
        packages: Array<string>
    };
    dockerfile: Array<string>;
}
interface ShellPackage {
    name: string;
    version: string;
    path: string;
}

export class DockerfileBuilderCLI {

    private _fullShellPackage: Map<string, ShellPackage>;

    static instance() {
        return new DockerfileBuilderCLI();
    }

    public async run() {
        await this._validate();
        await this._create();

        console.log('InitCLI complete.');
    }

    /**
     * 验证参数，数据，环境是否正确
     *
     * @returns {Promise<void>}
     * @private
     */
    private async _validate() {
        const jsonStat = await LibFs.stat(LibPath.join(process.cwd(), 'Dockerfile.json'));
        if (!jsonStat.isFile()) {
            throw new Error('Dockerfile.json not found!');
        }
    }

    /**
     * 创建Dockerfile
     *
     * @returns {Promise<void>}
     * @private
     */
    private async _create() {
        const configs: DockerConfigs = require(LibPath.join(process.cwd(), 'Dockerfile.json'));

        this._fullShellPackage = new Map();
        await this._recursionShellDependencies(configs.shell.store, configs.shell.packages);

        TplEngine.registerHelper('lcfirst', lcfirst);
        let DockerfileContent = TplEngine.render('Dockerfile', {
            alpine: configs.alpine,
            apk: configs.apk,
            npm: configs.npm,
            shell: Array.from(this._fullShellPackage.values()),
            dockerfile: configs.dockerfile,
        });
        await LibFs.writeFile(LibPath.join(process.cwd(), 'Dockerfile'), DockerfileContent);
    }

    private async _recursionShellDependencies(store: string, packages: Array<string>) {
        for (let key in packages) {
            const [name, version] = packages[key].split('@');

            const shellPath = LibPath.join(process.cwd(), store, name, version);
            const shellDependenciesPath = LibPath.join(shellPath, 'dependencies.json');

            if (LibFs.existsSync(shellDependenciesPath) && LibFs.statSync(shellDependenciesPath).isFile()) {
                await this._recursionShellDependencies(store, require(shellDependenciesPath));
                this._compareShellVersion({
                    name: name,
                    version: version,
                    path: calcRelativeDir(LibPath.relative(process.cwd(), shellPath))
                })
            } else {
                this._compareShellVersion({
                    name: name,
                    version: version,
                    path: calcRelativeDir(LibPath.relative(process.cwd(), shellPath))
                })
            }
        }
    }

    private _compareShellVersion(newShellPackage: ShellPackage) {
        if (!this._fullShellPackage.has(newShellPackage.name)) {
            this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            return;
        }

        const oldShellPackage = this._fullShellPackage.get(newShellPackage.name);
        let [nextMajor, nextMinor, nextPatch] = newShellPackage.version.split('.');
        let [curMajor, curMinor, curPatch] = oldShellPackage.version.split('.');

        if (nextMajor > curMajor) {
            this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            return;
        }

        if (nextMajor == curMajor) {
            if (nextMinor < curMinor || nextMinor == curMinor && nextPatch < curPatch) {
                // 依赖版本低于当前版本，不处理
            } else {
                this._fullShellPackage.set(newShellPackage.name, newShellPackage);
            }
        }
    }
}

DockerfileBuilderCLI.instance().run().catch((err: Error) => {
    console.log('error:', err.message);
});